GLOBAL.setmetatable(env, {
  __index = function(t, k) return GLOBAL.rawget(GLOBAL, k) end
})
modimport("scripts/apis.lua")
if not PREFAB_SKINS then
  local function OnFail()
    -- import translation
    utils.mod("scripts/TranslateLanguage")
    -- show error message
    utils.class("screens/scripterrorscreen", function(self)
      local text = STRINGS.UI.MAINSCREEN.MODTITLE
      if self.title then
        if self.title:GetString() == text then
          self.title:SetString(STRINGS.UI.MAINSCREEN.MODDSTENVNOTENABLED)
        end
      end
    end)
  end
  local function OnFound(name)
    print("now enable dst library " .. name)
    KnownModIndex:Enable(name)
    local entry = "../mods/" .. name .. "/modmain.lua"
    if kleifileexists and kleifileexists(entry) then
      package.path = "..\\mods\\" .. name .. "\\scripts\\?.lua;" .. package.path
      local newenv = CreateEnvironment(name)
      newenv.modname = name
      ModManager:InitializeModMain(name, newenv, "modmain.lua")
    end
  end
  print("The DST Library is not enabled!")
  -- try to enable it then.
  local dstmodname = {"DST Library", "联机库"}
  local flag = false
  for i, v in ipairs(dstmodname) do
    flag = flag or KnownModIndex:GetModActualName(v)
  end
  if flag then
    OnFound(flag)
  else
    OnFail()
    return
  end
end
modimport("scripts/entityapis.lua")
Assets = {
  Asset("ANIM", "anim/corner_dude_dst.zip"),
  Asset("ANIM", "anim/pig_house_dst.zip")
}
TUNING.USESKINFX = GetConfig("fx") == "true"
PrefabFiles = {"reskin_tool", "wardrobe", "dst_skin_fx"}
do
  local FxFiles = {
    "cane_candy_fx", "cane_sharp_fx", "cane_victorian_fx", "cane_harlequin_fx",
    "nightsword_wizard_fx", "nightsword_sharp_fx", "nightsword_lightsbane_fx",
    "nightsword_curve_fx", "nightsword_cotl_fx", "ds_skin_fx"
  }
  if TUNING.USESKINFX then PrefabFiles = table.union(PrefabFiles, FxFiles) end
end
local function talk(str, ...)
  local s
  if select("#", ...) > 0 then
    s = string.format(str, ...)
  else
    s = str
  end
  if ThePlayer.components then
    if ThePlayer.components.talker then ThePlayer.components.talker:Say(s) end
  end
end
expose("SetPlayerSkin", function(name)
  local b = name
  if not string.find(name, ThePlayer.prefab) then
    b = ThePlayer.prefab .. "_" .. (name or "none")
  end
  if PREFAB_SKINS[ThePlayer.prefab] then
    if table.contains(PREFAB_SKINS[ThePlayer.prefab], b) then
      if ThePlayer.components.skinner then
        if TheInventory:CheckOwnership(b) then
          ThePlayer.components.skinner:SetSkinName(b, nil, true)
          ThePlayer.components.skinner:SetSkinMode()
          talk(STRINGS.ANNOUNCESETSKINSUCCESS, GetSkinName(b))
        else
          talk(STRINGS.ANNOUNCESETSKINFAIL, GetSkinName(b))
        end
        return
      else
        print("Skinner component doesn't exist????")
      end
    else
      print("The Player has no such " .. b .. " skin!")
      print("full skin list:")
      local lst = {}
      for k, v in pairs(PREFAB_SKINS[ThePlayer.prefab]) do
        if TheInventory:CheckOwnership(v) then
          print(v)
          table.insert(lst,
                       string.format(STRINGS.ANNOUNCEEACHHINT, GetSkinName(v),
                                     string.sub(v, string.find(v, "_") + 1)))
        end
      end
      talk(STRINGS.ANNOUNCESETSKINHINT .. "\n" .. table.concat(lst, "\n"),
           ThePlayer:GetDisplayName())
    end
  else
    print("The Player has no skins!")
  end
end)
these_prefabs_are_correct = {
  "arrowsign_post", "axe", "backpack", "batbat", "bedroll_furry", "beebox",
  "beemine", "birdcage", "book_brimstone", "boomerang", "campfire", "cane",
  "cave_entrance", "cave_entrance_open", "chester", "chester_eyebone",
  "coldfirepit", "cookpot", "dragonflychest", "fence", "fence_gate_item",
  "fence_item", "firepit", "firesuppressor", "glommer", "glommerflower",
  "goldenaxe", "goldenpickaxe", "goldenpickaxe", "goldenpitchfork",
  "goldenshovel", "grass_umbrella", "gravestone", "hambat", "heatrock",
  "icebox", "insanityrock", "krampus_sack", "lighter", "lightning_rod",
  "minisign", "minisign_drawn", "minisign_item", "multitool_axe_pickaxe",
  "nightsword", "panflute", "pickaxe", "reflectivevest", "researchlab",
  "researchlab3", "reskin_tool", "resurrectionstone", "reviver", "saddle_basic",
  "shovel", "spear_wathgrithr", "telebase", "tent", "trap", "trap_teeth",
  "umbrella", "wall_ruins", "wall_ruins_item", "wall_stone", "wall_stone_item",
  "wall_wood", "wall_wood_item", "wardrobe", "whip", "winterometer", "wormhole",
  "pighouse", "researchlab4", "fence_gate", "abigail", -- reworked
  "abigail_flower", -- reworked
  "researchlab2", "bundle", "bundlewrap", "lantern"
}
these_are_wrong = {
  "piggyback", "bugnet", "razor", "spear", "featherfan", "ruins_bat", "torch",
  "pottedfern"
}
strange_shaped = {"lureplant", "rabbithouse", "treasurechest", "meatrack"}
need_some_code = {"wathgrithr_hat"}
LANTERN_COLORS = {
  lantern_tesla = {228 / 255, 46 / 255, 255 / 255},
  lantern_winter = {140 / 255, 205 / 255, 247 / 255},
  lantern_mummy = {102 / 255, 51 / 255, 153 / 255},
  lantern_crystal = {255 / 255, 175 / 255, 78 / 255},
  lantern_flower = {46 / 255, 139 / 255, 87 / 255},
  lantern_gothic = {255 / 255, 151 / 255, 27 / 255},
  lantern_retro = {255 / 255, 225 / 255, 0 / 255}
}
SKIN_RARITY = {
  Common = {0.718, 0.824, 0.851, 1}, -- B7D2D9 - a common item (eg t-shirt, plain gloves)
  Classy = {0.255, 0.314, 0.471, 1}, -- 415078 - an uncommon item (eg dress shoes, checkered trousers)
  Spiffy = {0.408, 0.271, 0.486, 1}, -- 68457C - a rare item (eg Trenchcoat)
  Distinguished = {0.729, 0.455, 0.647, 1}, -- BA74A5 - a very rare item (eg Tuxedo)
  Elegant = {0.741, 0.275, 0.275, 1}, -- BD4646 - an extremely rare item (eg rabbit pack, GoH base skins)
  HeirloomElegant = {0.933, 0.365, 0.251, 1}, -- EE5D40
  Character = {0.718, 0.824, 0.851, 1}, -- B7D2D9 - a character
  Timeless = {0.424, 0.757, 0.482, 1}, -- 6CC17B - not used
  Loyal = {0.635, 0.769, 0.435, 1}, -- A2C46F - a one-time giveaway (eg mini monument)
  ProofOfPurchase = {0.000, 0.478, 0.302, 1}, -- 007A4D
  Reward = {0.910, 0.592, 0.118, 1}, -- E8971E - a set bonus reward
  Event = {0.957, 0.769, 0.188, 1}, -- F4C430 - an event item
  Lustrous = {1.000, 1.000, 0.298, 1} -- FFFF4C - rarity modifier
  -- #40E0D0 reserved skin colour
}
-- Share colours
SKIN_RARITY.Complimentary = SKIN_RARITY.Common
SKIN_RARITY.HeirloomClassy = SKIN_RARITY.HeirloomElegant
SKIN_RARITY.HeirloomSpiffy = SKIN_RARITY.HeirloomElegant
SKIN_RARITY.HeirloomDistinguished = SKIN_RARITY.HeirloomElegant

utils.mod("scripts/loader.lua")
