TUNING.ITEMBUILD = TUNING.ITEMBUILD or {
  -- lantern's symbol is swap_lantern without _on/_off suffix
  swap_lantern = function(self)
    local lantern
    local ba = self.inst:GetBufferedAction()
    if ba then lantern = ba.target or ba.invobject end
    if not lantern then
      lantern = self.inst.components.inventory and
                  self.inst.components.inventory:GetEquippedItem(
                    EQUIPSLOTS.HANDS)
    end
    if not lantern or not lantern:GetSkinName() then return end
    local skin = lantern:GetSkinBuild()
    return skin
  end,
  -- fix RoG and vanilla
  pan_flute = function(self)
    local ba = self.inst:GetBufferedAction()
    return ba and ba.invobject and ba.invobject.flutebuild
  end,
  -- ALL
  swap_bedroll_furry = function(self)
    local ba = self.inst:GetBufferedAction()
    return ba and ba.invobject and ba.invobject:GetSkinBuild()
  end,
  fence_thin = function(self, b, s, o) return self.inst:GetSkinBuild() end,
  fence_gate_thin = function(self, b, s, o)
    local inst = self.inst.entity:GetParent()
    return inst:GetSkinBuild()
  end
}
TUNING.ITEMSYMBOL = TUNING.ITEMSYMBOL or {
  -- restricted to swap_lantern, not sawp_bottle_lantern
  swap_lantern_on = function(self, os, s, b)
    local lantern
    local ba = self.inst:GetBufferedAction()
    if ba then lantern = ba.target or ba.invobject end
    if not lantern then
      lantern = self.inst.components.inventory and
                  self.inst.components.inventory:GetEquippedItem(
                    EQUIPSLOTS.HANDS)
    end
    if not lantern or not lantern:GetSkinName() then return end
    if b:find("swap_lantern") == 1 then return "swap_lantern" end
  end,
  swap_lantern_off = function(self, os, s, b)
    local lantern
    local ba = self.inst:GetBufferedAction()
    if ba then lantern = ba.target or ba.invobject end
    if not lantern then
      lantern = self.inst.components.inventory and
                  self.inst.components.inventory:GetEquippedItem(
                    EQUIPSLOTS.HANDS)
    end
    if not lantern or not lantern:GetSkinName() then return end
    if b:find("swap_lantern") == 1 then return "swap_lantern" end
  end
}
local function Null() end
local Redirector = Class(function(self, inst)
  self.inst = inst
  self.animstate = inst.AnimState
  -- inst.AnimState = newproxy(true)
  -- self.proxy = getmetatable(inst.AnimState)
  self.meta = {
    SetBuild = function(_self, b)
      local nb = self:GetBuildName(b)
      self:print("SetBuild", b, nb)
      return self.animstate:SetBuild(nb)
    end,
    OverrideSymbol = function(_self, sym, build, overridesym)
      local nb = self:GetBuildName(build, sym, overridesym)
      local ny = self:GetSymbol(overridesym, build, sym)
      self:print("Redirector OverrideSymbol", sym, build, overridesym, nb, ny)
      return self.animstate:OverrideSymbol(sym, nb, ny)
    end
  }
  inst.AnimState = self.meta
  setmetatable(self.meta, {
    __index = function(_, k)
      if self.animstate[k] then
        return function(_, ...)
          return self.animstate[k](self.animstate, ...)
        end
      else
        return nil
      end
    end
  })
  -- self.proxy.__index = self.meta
  self.rules = {}
  self.Null = Null
end)
function Redirector:print(...)
  -- return print(...)
end
function Redirector:GetBuild(b, s, os)
  if TUNING.ITEMBUILD[b] then
    return FunctionOrValue(TUNING.ITEMBUILD[b], self, b, s, os) or b
  end
  if self.rules[b] then
    return FunctionOrValue(self.rules[b], self, b, s, os) or b
  end
  if self.rules[true] then
    for i, v in ipairs(self.rules[true]) do
      local ret = v(self, b, s, os)
      if ret ~= nil then return ret end
    end
  end
  return b
end
Redirector.GetBuildName = function(self, ...) return self:GetBuild(...) end
function Redirector:GetSymbol(os, s, b)
  if TUNING.ITEMSYMBOL[os] then
    return FunctionOrValue(TUNING.ITEMSYMBOL[os], self, os, s, b) or os
  end
  return os
end
function Redirector:RedirectBuild(from, to)
  if from == nil then return self:RedirectAny(to) end
  self.rules[from] = to
end
function Redirector:RedirectAny(fn)
  self.rules[true] = self.rules[true] or {}
  if not table.contains(self.rules[true], fn) then
    table.insert(self.rules[true], fn)
  end
end
function Redirector:Block(fnname, overridefn)
  if self.meta["_" .. fnname] then return false end
  self.meta["_" .. fnname] = self.meta[fnname]
  self.meta[fnname] = overridefn or self.Null
  return true
end
function Redirector:Unblock(fnname)
  if not self.meta["_" .. fnname] then return false end
  self.meta[fnname] = self.meta["_" .. fnname]
  self.meta["_" .. fnname] = nil
  return true
end

return Redirector
