local ParticleSpawner = Class(function(self, inst)
  self.inst = inst
  self.particles = {}
  self.managed = {}
  self.follower = self:SpawnFollower(inst)
  self.followerevents = {}
  -- inst.ps = self
  inst:ListenForEvent("onremove", function() self:KillParticles() end)
end)
local p = ParticleSpawner
function p:SpawnParticle()
  local inst = SpawnPrefab("ds_skin_fx")
  return inst
end
function p:SpawnShortLivedParticle(callback)
  local particle = self:SpawnParticle()
  particle:ListenForEvent("animover",
                          callback or function(inst) inst:Remove() end)
  return particle
end
function p:AttachTo(particle, parent)
  particle.Transform:SetPosition(0, 0, 0)
  particle.entity:SetParent(parent.entity)
end
function p:DetachFrom(particle) particle.entity:SetParent(nil) end
function p:TeleportTo(particle, parent)
  particle.Transform:SetPosition(parent.Transform:GetWorldPosition())
end
function p:Follow(particle, parent, symbol, x, y, z)
  self:TeleportTo(particle, parent)
  if not particle.Follower then particle.entity:AddFollower() end
  particle.Follower:FollowSymbol(parent.GUID, symbol, x or 0, y or 0, z or 0)
end
function p:Unfollow(particle) print("Error: Unable to unfollow in DS") end
function p:SpawnFollower(parent, symbol, offset)
  local inst = CreateEntity()
  inst:AddTag("FX")
  inst:AddTag("INTERIOR_LIMBO_IMMUNE")
  inst.entity:SetCanSleep(false)
  inst.persists = false
  inst.parent = parent
  inst.entity:AddTransform()
  if symbol then
    self:Follow(inst, parent, symbol, unpack(offset or {0, 0, 0}))
  elseif parent then
    self:AttachTo(inst, parent)
  end
  return inst
end
function p:EmitLoopedParticles(bank, build, anim, symbol, timing, amount,
                               reserved, postinit)
  local parent = self.follower
  for idx = 1, amount do
    for i, t in ipairs(timing) do
      parent:DoTaskInTime(t, function()
        local particle = self:SpawnParticle()
        particle._timing = t
        self:TeleportTo(particle, parent)
        particle.emitter = parent
        particle.owner = self.inst
        particle.paused = false
        local as = particle.AnimState
        as:SetBank(bank)
        particle._bank = bank
        as:SetBuild(build)
        particle._build = build
        for _, v in ipairs(symbol) do as:OverrideSymbol(unpack(v)) end
        particle._symbol = symbol
        if type(anim) == "table" then
          as:PlayAnimation(anim[i])
          particle._anim = anim[i]
        else
          as:PlayAnimation(anim)
          particle._anim = anim
        end
        table.insert(self.particles, particle)
        parent:ListenForEvent("onremove",
                              function() particle:PushEvent("candie") end)
        if postinit then postinit(particle) end
        if particle.onupdate then
          particle:ListenForEvent("animover", particle.onupdate)
        end
      end)
    end
  end
end
function p:EmitStaticParticle(bank, build, anim, symbol, postinit, followsymbol,
                              followxyz)
  local parent = self.inst
  local particle = self:SpawnParticle()
  if followsymbol then
    self:Follow(particle, parent, followsymbol, unpack(followxyz or {0, 0, 0}))
  else
    self:AttachTo(particle, parent)
  end
  particle.emitter = parent
  particle.parent = parent
  particle.owner = parent
  local as = particle.AnimState
  as:SetBank(bank)
  particle._bank = bank
  as:SetBuild(build)
  particle._build = build
  for _, v in ipairs(symbol) do as:OverrideSymbol(unpack(v)) end
  particle._symbol = symbol
  as:PlayAnimation(anim, true)
  particle._anim = anim
  table.insert(self.particles, particle)
  if postinit then postinit(particle) end
  if particle.onupdate then
    particle:ListenForEvent("animover", particle.onupdate)
  end
  particle:ListenForEvent("candie", particle.ondeath or
                            function() self:Kill(particle) end)
end
function p:EmitManagedParticle(prefab, maxlife, postinit)
  local inst = self.inst
  local inited = self.managedinited
  self.managedinited = true
  local function ondeath(i)
    i.parent = nil
    i.dying = GetTime()
    maxlife = maxlife or 0
    i:DoTaskInTime(maxlife, function()
      if i.dying then
        local thistime = GetTime() + 0.2 -- inaccurate
        if thistime - i.dying >= maxlife then self:Kill(i, self.managed) end
      end
    end)
  end
  local function onequip(_inst, data)
    local flag = false
    for i, oldfx in ipairs(self.managed) do
      if oldfx and oldfx:IsValid() then
        if oldfx.prefab == prefab then
          if oldfx.dying then
            oldfx.dying = nil
            oldfx.parent = self.inst
            flag = true
          else
            oldfx.candie = true
            oldfx:PushEvent("candie")
          end
        end
      end
    end
    if flag then return end
    local owner = data.owner or inst.components.inventoryitem.owner
    if owner then
      local fx = SpawnPrefab(prefab)
      if not fx then return end
      fx.parent = owner
      table.insert(self.managed, fx)
      self:Follow(fx, owner, inst.followsymbol or "swap_object",
                  unpack(inst.followxyz or {0, 0, 0}))
      fx:ListenForEvent("candie", ondeath)
    end
  end
  table.insert(self.followerevents, onequip)
  local function onunequip(_inst, data)
    for i, v in ipairs(self.managed) do v:PushEvent("candie") end
  end
  self.follower:ListenForEvent("equipped", onequip, inst)
  if not inited then
    self.follower:ListenForEvent("unequipped", onunequip, inst)
  end
  if postinit then postinit(self) end
end
function p:KillParticles()
  for i, v in ipairs(self.particles) do
    if v:IsValid() then
      v.candie = true
      v:PushEvent("candie")
    end
  end
  for i, v in ipairs(self.managed) do
    if v:IsValid() then
      v.candie = true
      v:PushEvent("candie")
    end
  end
  for i, v in ipairs(self.followerevents) do
    self.follower:RemoveEventCallback("equipped", v, self.inst)
  end
  self.particles = {}
  self.managed = {}
end
function p:Kill(particle, tbl)
  tbl = tbl or self.particles
  for i, v in ipairs(tbl) do
    if particle == v then
      table.remove(tbl, i)
      break
    end
  end
  if particle:IsValid() then particle:Remove() end
end
local function go(inst, state)
  if state ~= inst.currentstate then -- print(inst, "goto", state)
  end
  local fov = function(val) return FunctionOrValue(val, inst, state) end
  local statedef = inst.statemachine[state or ""]
  -- check the anim
  local as = inst.AnimState
  -- check if we should kill myself
  if inst.owner and not inst.owner:IsValid() then inst.shoulddie = true end
  if inst.shoulddie then
    if inst.paused or as:AnimDone() then
      inst:Remove()
    else
      inst:ListenForEvent("animover", inst.Remove)
    end
    return
  end
  if not statedef then
    -- just continue
    as:SetTime(0)
    return
  end
  -- check delay
  if statedef.delay then
    if inst.delayed then
      inst.delayed = nil
    else
      inst.delayed = true
      inst:DoTaskInTime(statedef.delay, inst.go, state)
      return
    end
  end
  if statedef.bank then as:SetBank(fov(statedef.bank)) end
  if statedef.build then as:SetBuild(fov(statedef.build)) end
  if statedef.anim then as:PlayAnimation(fov(statedef.anim)) end
  if statedef.symbol then
    for i, v in ipairs(statedef.symbol) do
      if not v[2] then
        as:ClearOverrideSymbol(v[1])
      else
        as:OverrideSymbol(unpack(v))
      end
    end
  end
  if statedef.time then
    as:SetTime(fov(statedef.time))
  elseif as:AnimDone() then
    as:SetTime(0)
  end
  local nextp = fov(statedef.paused)
  if nextp ~= nil and nextp ~= inst.paused then
    if nextp then
      as:Pause()
      inst:Hide()
    else
      -- care the timing
      if inst._timing then
        inst:DoTaskInTime(inst._timing, function()
          as:Resume()
          inst:Show()
        end)
      else
        as:Resume()
        inst:Show()
      end
    end
    inst.paused = nextp
  end
  inst.nextstate = nil
  fov(statedef.fn)
  inst.currentstate = state
  inst.nextstate = inst.nextstate or statedef.nextstate
  -- teleport
  local nextdef = inst.statemachine[inst.nextstate or ""] or {}
  if inst.emitter and not nextdef.noteleport and not statedef.static then
    p:TeleportTo(inst, inst.emitter)
  else
    --        print("no teleport", inst.nextstate, state)
  end
  -- check for death
  if inst.candie then
    if statedef.ondeath then
      fov(statedef.ondeath)
    else
      inst:Remove()
    end
  end
end
local function onupdate(inst)
  if inst.nextstate then
    inst:go(inst.nextstate)
  else
    inst:go(inst.currentstate)
  end
end
local function onevent(inst, event, state)
  if inst.paused == true then
    -- if it is paused, then directly go it
    inst:go(state)
  else
    -- wait for animover
    -- if there is an event then handle it
    -- the events are discarded one by one
    inst.nextstate = state
  end
end
function p:ContructStateMachine(inst, states, events)
  return function(particle)
    for k, v in pairs(events) do
      particle:ListenForEvent(k, function(...) particle:onevent(k, v, ...) end,
                              inst)
    end
    particle.statemachine = states
    particle.go = go
    particle.onupdate = onupdate
    particle.onevent = onevent
    particle:go("start")
  end
end
local defstates = {
  inventory = {paused = true},
  ground = {paused = true},
  equip = {
    paused = false,
    fn = function(inst)
      local owner = inst.owner
      if not owner then return end
      owner = owner.components.inventoryitem.owner
      if not owner then return end
      if not owner._emitter then
        owner._emitter = p:SpawnFollower(owner, inst.owner.followsymbol or
                                           "swap_object",
                                         inst.owner.followxyz or {0, 0, 0})
      end
      inst.emitter = owner._emitter
    end
  },
  start = {
    fn = function(inst)
      local owner = inst.owner
      if owner.inlimbo then
        inst.nextstate =
          owner.components.inventoryitem.isequipped and "equip" or "inventory"
      else
        inst.nextstate = "ground"
      end
      inst:DoTaskInTime(0, inst.go, inst.nextstate)
    end
  }
}
local defstates_static = {
  inventory = {paused = true, static = true, noteleport = true},
  ground = {paused = true, static = true, noteleport = true},
  equip = {
    static = true,
    paused = false,
    noteleport = true,
    fn = function(inst)
      local owner = inst.owner
      if not owner then return end
      owner = owner.components.inventoryitem.owner
      if not owner then return end
      p:Follow(inst, owner, inst.owner.followsymbol or "swap_object",
               unpack(inst.owner.followxyz or {}))
    end
  },
  start = {
    fn = function(inst)
      local owner = inst.owner
      inst.AnimState:PlayAnimation(inst._anim, true)
      if owner.inlimbo then
        inst.nextstate =
          owner.components.inventoryitem.isequipped and "equip" or "inventory"
      else
        inst.nextstate = "ground"
      end
      inst:DoTaskInTime(0, inst.go, inst.nextstate)
    end
  }
}
local defstates_ground = {
  inventory = {paused = true},
  ground = {paused = false},
  start = {
    fn = function(inst)
      inst.emitter = inst.owner
      if inst.owner.inlimbo then
        inst.nextstate = "inventory"
      else
        if inst.statemachine.ground_loop then
          inst.nextstate = "ground_loop"
        else
          inst.nextstate = "ground"
        end
      end
      inst:DoTaskInTime(0, inst.go, inst.nextstate)
    end
  }
}
local defstates_trail = {
  inventory = {paused = true},
  ground = {paused = true},
  equip = {
    paused = false,
    fn = function(inst)
      inst.emitter = nil
      local owner = inst.owner
      if not owner then return end
      owner = owner.components.inventoryitem.owner
      if not owner then return end
      inst.emitter = owner
    end
  },
  start = {
    fn = function(inst)
      local owner = inst.owner
      if owner.inlimbo then
        inst.nextstate =
          owner.components.inventoryitem.isequipped and "equip" or "inventory"
      else
        inst.nextstate = "ground"
      end
      inst:DoPeriodicTask(5, function()
        -- random delay
        inst._timing = math.random(60) * FRAMES
      end)
      inst:DoTaskInTime(0, owner.go, owner.nextstate)
    end
  }
}
local defstates_ground_static = {
  inventory = {paused = true, static = true, noteleport = true},
  ground = {paused = false, static = true, noteleport = true},
  start = {
    noteleport = true,
    static = true,
    fn = function(inst)
      local owner = inst.owner
      p:AttachTo(inst, owner)
      inst.AnimState:PlayAnimation(inst._anim, true)
      if owner.inlimbo then
        inst.nextstate = "inventory"
      else
        if inst.statemachine.ground_loop then
          inst.nextstate = "ground_loop"
        else
          inst.nextstate = "ground"
        end
      end
      inst:DoTaskInTime(0, inst.go, inst.nextstate)
    end
  }
}
local events_item = {
  enterlimbo = "inventory",
  onputininventory = "inventory",
  onpickup = "inventory",
  ondropped = "ground",
  equipped = "equip"
  -- unequipped
}
local events_ground = {
  enterlimbo = "inventory",
  onputininventory = "inventory",
  onpickup = "inventory",
  ondropped = "ground",
  equipped = "inventory"
}
function p:ConstructHeldItemStateMachine(inst, states)
  return self:ConstructPresetStateMachine(inst, states, defstates, events_item)
end
function p:ConstructGroundItemStateMachine(inst, states)
  return self:ConstructPresetStateMachine(inst, states, defstates_ground,
                                          events_ground)
end
function p:ConstructStaticHeldItemStateMachine(inst, states)
  return self:ConstructPresetStateMachine(inst, states, defstates_static,
                                          events_item)
end
function p:ConstructStaticGroundStateMachine(inst, states)
  return self:ConstructPresetStateMachine(inst, states, defstates_ground_static,
                                          events_ground)
end
function p:ConstructTrailStateMachine(inst, states)
  return self:ConstructPresetStateMachine(inst, states, defstates_trail,
                                          events_item)
end
function p:ConstructPresetStateMachine(inst, states, presetstate, presetevent)
  local allstates = {}
  table.mergeinto(allstates, presetstate)
  if states then table.mergeinto(allstates, states) end
  return self:ContructStateMachine(inst, allstates, presetevent)
end
function p:HandleBurnable(particle, parent)
  local function ondeath() self:Kill(particle) end
  particle:ListenForEvent("burntup", ondeath, parent)
  particle:ListenForEvent("onburntup", ondeath, parent)
end
return ParticleSpawner
