return {
  ACTIONS = {ActionHandler(ACTIONS.CHANGEIN, "usewardrobe")},
  PATCHEDACTIONS = {
    CASTSPELL = function(action)
      local function new(inst, action)
        return action.invobject ~= nil and ((action.invobject:HasTag("gnarwail_horn") and "play_gnarwail_horn")
                 or (action.invobject:HasTag("guitar") and "play_strum")
                 or (action.invobject:HasTag("cointosscast") and "cointosscastspell")
                 or (action.invobject:HasTag("quickcast") and "quickcastspell")
                 or (action.invobject:HasTag("veryquickcast") and "veryquickcastspell")) -- trimmed
      end
      local old = action.deststate
      action.deststate = function(inst, action) return new(inst, action) or FunctionOrValue(old, inst, action) end
    end
  },
  STATES = {
    State {
      name = "veryquickcastspell",
      tags = {"doing", "canrotate", "busy"},

      onenter = function(inst)
        -- inst.components.locomotor:Stop()--deleted
        if inst.components.rider:IsRiding() then
          inst.AnimState:PlayAnimation("atk_pre") -- changed
          inst.AnimState:PushAnimation("atk", false) -- changed
        else
          -- inst.AnimState:PlayAnimation("atk_pre")--changed
          inst.AnimState:PlayAnimation("atk") -- changed
        end
        inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
      end,

      timeline = {
        TimeEvent(9 * FRAMES, function(inst)
          inst:PerformBufferedAction()
          inst.sg:RemoveStateTag("busy")
        end)
      },

      events = {
        EventHandler("animqueueover", function(inst)
          if inst.AnimState:AnimDone() then inst.sg:GoToState("idle") end
        end)
      }
    },
    State {
      name = "changeinwardrobe",
      tags = {"inwardrobe", "busy", "nopredict", "silentmorph"},

      onenter = function(inst, delay)
        -- This state is only valid as a substate of openwardrobe
        inst:Hide()
        inst.DynamicShadow:Enable(false)
        inst.sg.statemem.isplayerhidden = true

        inst.sg:SetTimeout(delay)
        if inst.components.health then inst.components.health:SetInvincible(true) end
      end,

      ontimeout = function(inst)
        inst.AnimState:PlayAnimation("jumpout_wardrobe")
        inst:Show()
        inst.DynamicShadow:Enable(true)
        if inst.components.health then inst.components.health:SetInvincible(false) end
        inst.sg.statemem.isplayerhidden = nil
        inst.sg.statemem.task = inst:DoTaskInTime(4.5 * FRAMES, function()
          inst.sg.statemem.task = nil
          inst.SoundEmitter:PlaySound("dontstarve/movement/bodyfall_dirt")
        end)
      end,

      events = {
        EventHandler("animover", function(inst)
          if not inst.sg.statemem.isplayerhidden and inst.AnimState:AnimDone() then inst.sg:GoToState("idle") end
        end)
      },

      onexit = function(inst)
        if inst.sg.statemem.task ~= nil then
          inst.sg.statemem.task:Cancel()
          inst.sg.statemem.task = nil
        end
        if inst.components.health then inst.components.health:SetInvincible(false) end
        if inst.sg.statemem.isplayerhidden then
          inst:Show()
          inst.DynamicShadow:Enable(true)
          inst.sg.statemem.isplayerhidden = nil
        end
        -- Cleanup from openwardobe state
        if inst.components.playercontroller ~= nil then
          inst.components.playercontroller:EnableMapControls(true)
          inst.components.playercontroller:Enable(true)
        end
        inst.components.inventory:Show()
        -- inst:ShowActions(true)--#FIXME
        if not inst.sg.statemem.isclosingwardrobe then
          inst.sg.statemem.isclosingwardrobe = true
          POPUPS.WARDROBE:Close(inst)
        end
      end
    },
    State {
      name = "usewardrobe",
      tags = {"doing"},

      onenter = function(inst)
        inst.components.locomotor:Stop()
        inst.AnimState:PlayAnimation("give")
        inst.AnimState:PushAnimation("give_pst", false)
      end,

      timeline = {TimeEvent(13 * FRAMES, function(inst) inst:PerformBufferedAction() end)},

      events = {
        EventHandler("animqueueover", function(inst)
          if inst.AnimState:AnimDone() then inst.sg:GoToState("idle") end
        end)
      }
    },
    State {
      name = "openwardrobe",
      tags = {"inwardrobe", "busy", "pausepredict"},

      onenter = function(inst, data)
        inst.sg.statemem.isopeninggift = data.openinggift
        if not inst.sg.statemem.isopeninggift then
          inst.components.locomotor:Stop()
          inst.components.locomotor:Clear()
          -- inst:ClearBufferedAction()

          inst.AnimState:PlayAnimation("idle_wardrobe1_pre")
          inst.AnimState:PushAnimation("idle_wardrobe1_loop", true)

          if inst.components.playercontroller ~= nil then
            inst.components.playercontroller:RemotePausePrediction()
            inst.components.playercontroller:EnableMapControls(false)
            inst.components.playercontroller:Enable(false)
          end
          inst.components.inventory:Hide()
          inst:PushEvent("ms_closepopups")
          -- inst:ShowActions(false)--#TODO:investigate
        elseif inst.components.playercontroller ~= nil then
          inst.components.playercontroller:RemotePausePrediction()
        end
        if data.target and data.target.components.groomer then
          assert(data.target.components.groomer.occupant, "Grooming station had not occupant")
          inst:ShowPopUp(POPUPS.GROOMER, true, data.target.components.groomer.occupant, inst)
        else
          inst:ShowPopUp(POPUPS.WARDROBE, true, data.target)
          if inst.components.health then inst.components.health:SetInvincible(true) end
        end
      end,

      events = {
        EventHandler("firedamage", function(inst)
          if inst.sg.statemem.isopeninggift then
            inst.AnimState:PlayAnimation("gift_open_pst")
            inst.sg:GoToState("idle", true)
          else
            inst.sg:GoToState("idle")
          end
          if inst.components.talker ~= nil then
            inst.components.talker:Say(GetString(inst, "ANNOUNCE_NOWARDROBEONFIRE"))
          end
        end)
      },

      onexit = function(inst)
        -- inst:ShowPopUp(POPUPS.GROOMER, false)--#TODO
        inst:ShowPopUp(POPUPS.WARDROBE, false)
        if not inst.sg.statemem.ischanging then
            if inst.components.health then inst.components.health:SetInvincible(false) end
          if inst.components.playercontroller ~= nil then
            inst.components.playercontroller:EnableMapControls(true)
            inst.components.playercontroller:Enable(true)
          end
          inst.components.inventory:Show()
          -- inst:ShowActions(true)--#FIXME
          if not inst.sg.statemem.isclosingwardrobe then
            inst.sg.statemem.isclosingwardrobe = true
            POPUPS.WARDROBE:Close(inst)
          end
        end
      end
    }
  }
}
