local _Widget = require"widgets/widget"
return function(Widget)
    function Widget:IsEditing()
        -- recursive check to see if anything has text edit focus
        if self.editing then return true end

        for k, v in pairs(self.children) do if v:IsEditing() then return true end end

        return false
    end
    local oldKillAllChildren = Widget.KillAllChildren
    function Widget:KillAllChildren()
        oldKillAllChildren(self)
        self:ClearHoverText()
    end
    function Widget:Hide()
        self.inst.entity:Hide(false)
        local was_visible = self.shown == true
        self.shown = false
        self:OnHide(was_visible)
    end
    function Widget:Show()
        self.inst.entity:Show(false)
        local was_hidden = self.shown == false
        self.shown = true
        self:OnShow(was_hidden)
    end
    function Widget:GetPositionXYZ() return self.inst.UITransform:GetLocalPosition() end
    function Widget:SetHoverText(text, params)
        if text and text ~= "" then
            if not self.hovertext then
                local ImageButton = require"widgets/imagebutton"
                local Text = require"widgets/text"
                local Image = require"widgets/image"

                if params == nil then params = {} end

                if params.attach_to_parent ~= nil then
                    self.hovertext_root = params.attach_to_parent:AddChild(Widget("hovertext_root"))
                else
                    self.hovertext_root = _Widget("hovertext_root")
                    self.hovertext_root.global_widget = true
                    self.hovertext_root:SetScaleMode(SCALEMODE_PROPORTIONAL)
                end
                self.hovertext_root:Hide()

                if params.bg ~= false then
                    self.hovertext_bg = self.hovertext_root:AddChild(
                        Image(params.bg_atlas or "images/frontend.xml", params.bg_texture or "scribble_black.tex"))
                    self.hovertext_bg:SetTint(1, 1, 1, .8)
                    self.hovertext_bg:SetClickable(false)
                end

                self.hovertext = self.hovertext_root:AddChild(
                    Text(params.font or NEWFONT_OUTLINE, params.font_size or 22, text))
                self.hovertext:SetClickable(false)
                self.hovertext:SetScale(1.1, 1.1)

                if params.region_h ~= nil or params.region_w ~= nil then
                    self.hovertext:SetRegionSize(params.region_w or 1000, params.region_h or 40)
                end

                if params.wordwrap ~= nil then
                    -- print("Enabling word wrap", params.wordwrap)
                    self.hovertext:EnableWordWrap(params.wordwrap)
                end

                if params.colour then self.hovertext:SetColour(params.colour) end

                if params.bg == nil or params.bg == true then
                    local w, h = self.hovertext:GetRegionSize()
                    self.hovertext_bg:SetSize(w * 1.5, h * 2.0)
                end

                local hover_parent = self.text or self
                if hover_parent.GetString ~= nil and hover_parent:GetString() ~= "" then
                    -- Note(Peter): This block is here because Text widgets don't receive OnGainFocus calls.
                    self.hover = hover_parent:AddChild(ImageButton("images/dstui.xml", "blank.tex", "blank.tex",
                        "blank.tex", nil, nil, {1, 1}, {0, 0}))
                    self.hover.image:ScaleToSize(hover_parent:GetRegionSize())

                    self.hover.OnGainFocus = function()
                        local world_pos = self:GetWorldPosition()
                        local x_pos = world_pos.x + (params.offset_x or 0)
                        local y_pos = world_pos.y + (params.offset_y or 26)
                        self.hovertext_root:SetPosition(x_pos, y_pos)
                        self.hovertext_root:Show()
                    end
                    self.hover.OnLoseFocus = function() self.hovertext_root:Hide() end
                else
                    self._OnGainFocus = self.OnGainFocus -- save these fns so we can undo the hovertext on focus when clearing the text
                    self._OnLoseFocus = self.OnLoseFocus

                    self.OnGainFocus = function()
                        if params.attach_to_parent ~= nil then
                            local world_pos = self:GetWorldPosition() - params.attach_to_parent:GetWorldPosition()
                            local parent_scale = params.attach_to_parent:GetScale()

                            local x_pos = world_pos.x / parent_scale.x + (params.offset_x or 0)
                            local y_pos = world_pos.y / parent_scale.y + (params.offset_y or 26)
                            self.hovertext_root:SetPosition(x_pos, y_pos)

                            self.hovertext_root:MoveToFront()
                        else
                            local world_pos = self:GetWorldPosition()
                            local x_pos = world_pos.x + (params.offset_x or 0)
                            local y_pos = world_pos.y + (params.offset_y or 26)
                            self.hovertext_root:SetPosition(x_pos, y_pos)
                        end
                        self.hovertext_root:Show()

                        self._OnGainFocus(self)
                    end
                    self.OnLoseFocus = function()
                        self.hovertext_root:Hide()
                        self._OnLoseFocus(self)
                    end
                end
            else
                self.hovertext:SetString(text)
                if params and params.colour then self.hovertext:SetColour(params.colour) end
                if self.hovertext_bg then
                    local w, h = self.hovertext:GetRegionSize()
                    self.hovertext_bg:SetSize(w * 1.5, h * 2.0)
                end
            end
        end
    end

    function Widget:ClearHoverText()
        if self.hovertext_root ~= nil then
            self.hovertext_root:Kill()
            self.hovertext_root = nil
            self.hovertext = nil
            self.hovertext_bg = nil

            if self._OnGainFocus then
                self.OnGainFocus = self._OnGainFocus
                self.OnLoseFocus = self._OnLoseFocus

                self._OnGainFocus = nil
                self._OnLoseFocus = nil
            end
        end
        if self.hover ~= nil then
            self.hover:Kill()
            self.hover = nil
        end
    end

    function Widget:SetOnGainFocus(fn) self.ongainfocusfn = fn end

    function Widget:SetOnLoseFocus(fn) self.onlosefocusfn = fn end
    local oldClearFocusDirs = Widget.ClearFocusDirs
    function Widget:ClearFocusDirs()
        oldClearFocusDirs(self)
        self.focus_flow_args = {}
        self.next_in_tab_order = nil
    end

    function Widget:GetFocusChild()
        if self.focus then for k, v in pairs(self.children) do if v.focus then return v end end end
        return nil
    end

    function Widget:ClearFocus()
        if self.focus then
            self.focus = false
            if self.OnLoseFocus then self:OnLoseFocus() end
            if self.onlosefocusfn then self.onlosefocusfn() end
            for k, v in pairs(self.children) do if v.focus then v:ClearFocus() end end
        end
    end

    function Widget:SetFocusFromChild(from_child)
        if self.parent == nil and not self.is_screen then
            -- print(
            --  "Warning: Widget:SetFocusFromChild is happening on a widget outside of the screen/widget hierachy. This will cause focus moves to fail. Is ",
            -- self.name, "not a screen?")
            -- print(debugstack())
        end
        for k, v in pairs(self.children) do if v ~= from_child and v.focus then v:ClearFocus() end end

        if not self.focus then
            self.focus = true
            if self.OnGainFocus then self:OnGainFocus() end

            if self.ongainfocusfn then self.ongainfocusfn() end

            if self.parent then self.parent:SetFocusFromChild(self) end
        end
    end

    function Widget:SetFocus()
        --  print ("SET FOCUS ", self)
        local focus_forward = FunctionOrValue(self.focus_forward)
        if focus_forward then
            focus_forward:SetFocus()
            return
        end

        if not self.focus then
            self.focus = true

            if self.OnGainFocus then self:OnGainFocus() end

            if self.ongainfocusfn then self.ongainfocusfn() end

            if self.parent then self.parent:SetFocusFromChild(self) end
        end

        for k, v in pairs(self.children) do v:ClearFocus() end

        -- print(debugstack())
    end
end
