local texture = resolvefilepath "fx/confetti.tex"
local shader = resolvefilepath "shaders/vfx_particle.ksh"
local COLOUR_ENVELOPE_NAME = "cane_confetti_colourenvelope"
local SCALE_ENVELOPE_NAME = "cane_confetti_scaleenvelope"
local assets = {Asset("IMAGE", texture), Asset("SHADER", shader)}

local function IntColour(r, g, b, a)
    return {r / 255.0, g / 255.0, b / 255.0, a / 255.0}
end
local num_emitters = 4
local function InitEnvelope()
    local envs = {}

    EnvelopeManager:AddColourEnvelope(COLOUR_ENVELOPE_NAME .. 0, {
        {0, IntColour(255, 0, 0, 255)},
        {0.5, IntColour(255, 0, 0, 255)},
        {1, IntColour(255, 0, 0, 0)}
    })

    EnvelopeManager:AddColourEnvelope(COLOUR_ENVELOPE_NAME .. 1, {
        {0, IntColour(0, 200, 0, 255)},
        {0.5, IntColour(0, 200, 0, 255)},
        {1, IntColour(0, 200, 0, 0)}
    })

    EnvelopeManager:AddColourEnvelope(COLOUR_ENVELOPE_NAME .. 2, {
        {0, IntColour(21, 85, 203, 255)},
        {0.5, IntColour(21, 85, 203, 255)},
        {1, IntColour(21, 85, 203, 0)}
    })

    EnvelopeManager:AddColourEnvelope(COLOUR_ENVELOPE_NAME .. 3, {
        {0, IntColour(255, 255, 255, 255)},
        {0.5, IntColour(255, 255, 255, 255)},
        {1, IntColour(255, 255, 255, 0)}
    })

    EnvelopeManager:AddColourEnvelope(COLOUR_ENVELOPE_NAME .. 4, {
        {0, IntColour(233, 224, 44, 255)},
        {0.5, IntColour(233, 224, 44, 255)},
        {1, IntColour(233, 224, 44, 0)}
    })

    local max_scale = .7
    local end_scale = .4
    local t = 0
    local step = .2
    while t + step < 1 do
        local s = Lerp(max_scale, end_scale, Clamp(2 * t - 0.5, 0, 1))
        table.insert(envs, {t, {s * 0.25, s}})
        t = t + step

        local s = Lerp(max_scale, end_scale, Clamp(2 * t - 0.5, 0, 1))
        table.insert(envs, {t, {s, s * 0.2}})
        t = t + step
    end
    table.insert(envs, {1, {max_scale, max_scale * 0.6}})

    EnvelopeManager:AddVector2Envelope(SCALE_ENVELOPE_NAME, envs)

    InitEnvelope = nil
    IntColour = nil
end
local MAX_LIFETIME = 1.5

local function emit_confetti_fn(emitter, sphere_emitter)
    local vx, vy, vz = .012 * UnitRand(), 0, .012 * UnitRand()
    local lifetime = MAX_LIFETIME * (.5 + UnitRand() * .5)
    local px, py, pz = sphere_emitter()
    vy = vy + 0.5 -- apply upward force to all confetti
    local angle = math.random() * 360
    local uv_offset = math.random(0, 3) * .25
    local ang_vel = (UnitRand() - 1) * 5
    emitter:AddRotatingParticleUV(lifetime, px, py, pz, vx, vy, vz, angle, ang_vel, uv_offset, 0)
end
local function fn(i)
    local inst = Prefabs.ds_skin_fx_noanim.fn()
    local emitter = inst.entity:AddParticleEmitter()
    if InitEnvelope ~= nil then InitEnvelope() end
    emitter:SetRenderResources(texture, shader)
    emitter:SetRotationStatus(true)
    emitter:SetUVFrameSize(.25, 1)
    emitter:SetMaxNumParticles(200 / num_emitters)
    emitter:SetMaxLifetime(MAX_LIFETIME)
    emitter:SetColourEnvelope(COLOUR_ENVELOPE_NAME .. i)
    emitter:SetScaleEnvelope(SCALE_ENVELOPE_NAME)
    emitter:SetBlendMode(BLENDMODE.Premultiplied)
    emitter:EnableBloomPass(true)
    emitter:SetSortOrder(0)
    emitter:SetAcceleration(0, -0.4, 0)
    emitter:SetDragCoefficient(.1)
    local tick_time = TheSim:GetTickTime()

    local sparkle_desired_pps_low = 0
    local sparkle_desired_pps_high = 15
    local low_per_tick = sparkle_desired_pps_low * tick_time
    local high_per_tick = sparkle_desired_pps_high * tick_time
    local num_to_emit = 0

    local sphere_emitter = CreateSphereEmitter(.25)
    inst.last_pos = inst:GetPosition()
    EmitterManager:AddEmitter(inst, nil, function()
        local dist_moved = inst:GetPosition() - inst.last_pos
        local move = dist_moved:Length()
        move = math.clamp((move - 0.2) * 10, 0, 1)
        local per_tick = Lerp(low_per_tick, high_per_tick, move)
        inst.last_pos = inst:GetPosition()
        num_to_emit = num_to_emit + per_tick * math.random() * 3
        if inst.dying then num_to_emit = 0 end
        while num_to_emit > 1 do
            emit_confetti_fn(emitter, sphere_emitter)
            num_to_emit = num_to_emit - 1
        end
    end)
    return inst
end
local prefs = {}
for i = 0, num_emitters do
    table.insert(prefs, Prefab("cane_harlequin_fx" .. i, function()
        return fn(i)
    end, assets))
end
return unpack(prefs)
