local r = resolvefilepath
local ANIM_SMOKE_TEXTURE = r "fx/animsmoke.tex"
local ANIM_SPARK_TEXTURE = r "fx/animsponge.tex"

local REVEAL_SHADER = r "shaders/vfx_particle_reveal.ksh"

local COLOUR_ENVELOPE_NAME_SPARK = "nightsword_wizard_colourenvelope_spark"
local SCALE_ENVELOPE_NAME_SPARK = "nightsword_wizard_scaleenvelope_spark"

local COLOUR_ENVELOPE_NAME_SMOKE = "nightsword_wizard_colourenvelope_smoke"
local SCALE_ENVELOPE_NAME_SMOKE = "nightsword_wizard_scaleenvelope_smoke"

local assets = {Asset("IMAGE", ANIM_SMOKE_TEXTURE), Asset("IMAGE", ANIM_SPARK_TEXTURE), Asset("SHADER", REVEAL_SHADER)}

--------------------------------------------------------------------------

local function IntColour(r, g, b, a)
    return {r / 255, g / 255, b / 255, a / 255}
end

local function InitEnvelope()
    -- SMOKE
    EnvelopeManager:AddColourEnvelope(COLOUR_ENVELOPE_NAME_SMOKE, {
        {0, IntColour(10, 10, 10, 0)},
        {.3, IntColour(10, 10, 10, 175)},
        {.52, IntColour(10, 10, 10, 90)},
        {1, IntColour(10, 10, 10, 0)}
    })

    local smoke_max_scale = 0.5
    EnvelopeManager:AddVector2Envelope(SCALE_ENVELOPE_NAME_SMOKE, {
        {0, {smoke_max_scale * .5, smoke_max_scale * .5}},
        {1, {smoke_max_scale, smoke_max_scale}}
    })

    -- SPARK
    EnvelopeManager:AddColourEnvelope(COLOUR_ENVELOPE_NAME_SPARK, {
        {0.0, IntColour(100, 100, 100, 0)},
        {0.2, IntColour(50, 50, 50, 200)},
        {1.0, IntColour(0, 0, 0, 100)}
    })
    local spark_max_scale = 0.6
    EnvelopeManager:AddVector2Envelope(SCALE_ENVELOPE_NAME_SPARK, {
        {0.0, {spark_max_scale, spark_max_scale}},
        {1.0, {spark_max_scale, spark_max_scale}}
    })

    InitEnvelope = nil
    IntColour = nil
end

--------------------------------------------------------------------------

local SMOKE_MAX_LIFETIME = 0.75
local SPARK_MAX_LIFETIME = 1.5

local function emit_smoke_fn(effect, sphere_emitter, adjust_vec)
    local vx, vy, vz = .12 * UnitRand(), -.015 + .02 * UnitRand(), .12 * UnitRand()
    local lifetime = SMOKE_MAX_LIFETIME * (.9 + math.random() * .1)
    local px, py, pz = sphere_emitter()
    if adjust_vec ~= nil then
        px = px + adjust_vec.x
        py = py + adjust_vec.y
        pz = pz + adjust_vec.z
    end

    effect:AddRotatingParticle(lifetime, -- lifetime
    px, py + .5, pz, -- position
    vx, vy, vz, -- velocity
    math.random() * 360, -- angle
    UnitRand() * 2 -- angle velocity
    )
end

local function emit_spark_fn(effect, sphere_emitter, adjust_vec)
    local vx, vy, vz = .005 * UnitRand(), 0, .005 * UnitRand()
    local lifetime = SPARK_MAX_LIFETIME * (0.7 + math.random() * .3)
    local px, py, pz = sphere_emitter()
    if adjust_vec ~= nil then
        px = px + adjust_vec.x
        py = py + adjust_vec.y
        pz = pz + adjust_vec.z
    end

    effect:AddRotatingParticle(lifetime, -- lifetime
    px, py, pz, -- position
    vx, vy, vz, -- velocity
    math.random() * 360, -- angle
    UnitRand() * 1 -- angle velocity
    )
end
local function smoke()
    local inst = Prefabs.ds_skin_fx_noanim.fn()
    if InitEnvelope ~= nil then InitEnvelope() end

    local effect = inst.entity:AddParticleEmitter()
    effect:InitEmitters(2)

    -- SMOKE
    effect:SetRenderResources(ANIM_SMOKE_TEXTURE, REVEAL_SHADER)
    effect:SetMaxNumParticles(32)
    effect:SetRotationStatus(true)
    effect:SetMaxLifetime(SMOKE_MAX_LIFETIME)
    effect:SetColourEnvelope(COLOUR_ENVELOPE_NAME_SMOKE)
    effect:SetScaleEnvelope(SCALE_ENVELOPE_NAME_SMOKE)
    effect:SetBlendMode(BLENDMODE.AlphaBlended)
    effect:SetSortOrder(0)
    effect:SetSortOffset(0)
    effect:SetRadius(3) -- only needed on a single emitter
    effect:SetDragCoefficient(.1)

    local burst_state = 0

    local smoke_sphere_emitter = CreateSphereEmitter(.2)

    EmitterManager:AddEmitter(inst, nil, function()
        local parent = inst.parent
        if parent ~= nil then
            local mount = parent.components.rider ~= nil and parent.components.rider:GetMount() or nil
            if mount == nil then

                local attack_playing = parent.AnimState:IsCurrentAnimation("atk")
                local anim_time = parent.AnimState:GetCurrentAnimationTime()

                if attack_playing then
                    if anim_time > 0.13 and burst_state == 0 then
                        burst_state = 1 -- do burst
                    elseif burst_state == 2 then
                        burst_state = 0 -- added
                    end
                else
                    burst_state = 0 -- wait for atk anim
                end

                if burst_state == 1 then
                    burst_state = 2 -- wait for new atk
                    local num_to_emit_smoke = 5
                    local num_to_emit_spark = 3

                    local adjust_vec = nil
                    if parent.AnimState:GetCurrentFacing() == 1 then
                        -- Do custom positioning
                        adjust_vec = TheCamera:GetRightVec() * 0.75 - TheCamera:GetDownVec() * 2.6
                    end

                    while num_to_emit_smoke > 0 do
                        emit_smoke_fn(effect, smoke_sphere_emitter, adjust_vec)
                        num_to_emit_smoke = num_to_emit_smoke - 1
                    end
                end

            end
        end
    end)

    return inst
end
local function spark()
    local inst = Prefabs.ds_skin_fx_noanim.fn()
    if InitEnvelope ~= nil then InitEnvelope() end
    local effect = inst.entity:AddParticleEmitter()
    effect:InitEmitters(2)

    -- SPARK
    effect:SetRenderResources(ANIM_SPARK_TEXTURE, REVEAL_SHADER)
    effect:SetMaxNumParticles(32)
    effect:SetRotationStatus(true)
    effect:SetMaxLifetime(SPARK_MAX_LIFETIME)
    effect:SetColourEnvelope(COLOUR_ENVELOPE_NAME_SPARK)
    effect:SetScaleEnvelope(SCALE_ENVELOPE_NAME_SPARK)
    effect:SetBlendMode(BLENDMODE.AlphaBlended)
    effect:SetUVFrameSize(1, 1)
    effect:SetSortOrder(0)
    effect:SetSortOffset(0)
    effect:SetDragCoefficient(1)

    -----------------------------------------------------
    local burst_state = 0

    local spark_sphere_emitter = CreateSphereEmitter(.5)

    EmitterManager:AddEmitter(inst, nil, function()
        local parent = inst.parent
        if parent ~= nil then
            local mount = parent.components.rider ~= nil and parent.components.rider:GetMount() or nil
            if mount == nil then

                local attack_playing = parent.AnimState:IsCurrentAnimation("atk")
                local anim_time = parent.AnimState:GetCurrentAnimationTime()

                if attack_playing then
                    if anim_time > 0.13 and burst_state == 0 then
                        burst_state = 1 -- do burst
                    elseif burst_state == 2 then
                        burst_state = 0 -- added
                    end
                else
                    burst_state = 0 -- wait for atk anim
                end

                if burst_state == 1 then
                    burst_state = 2 -- wait for new atk
                    local num_to_emit_spark = 3

                    local adjust_vec = nil
                    if parent.AnimState:GetCurrentFacing() == 1 then
                        -- Do custom positioning
                        adjust_vec = TheCamera:GetRightVec() * 0.75 - TheCamera:GetDownVec() * 2.6
                    end

                    while num_to_emit_spark > 0 do
                        emit_spark_fn(effect, spark_sphere_emitter, adjust_vec)
                        num_to_emit_spark = num_to_emit_spark - 1
                    end
                end

            end
        end
    end)

    return inst
end
local prefs = {smoke, spark}
for i, v in ipairs(prefs) do prefs[i] = Prefab("nightsword_wizard_fx" .. i, v, assets) end
return unpack(prefs)

