-- This is a rewritten version of DST skin spinner, because the spinner widget is largely different.
local Image = require "widgets/image"
local Widget = require "widgets/widget"
local Spinner = require "widgets/dstspinner"

local textures = {
    arrow_left_normal = "crafting_inventory_arrow_l_idle.tex",
    arrow_left_over = "crafting_inventory_arrow_l_hl.tex",
    arrow_left_disabled = "arrow_left_disabled.tex",
    arrow_left_down = "crafting_inventory_arrow_l_hl.tex",
    arrow_right_normal = "crafting_inventory_arrow_r_idle.tex",
    arrow_right_over = "crafting_inventory_arrow_r_hl.tex",
    arrow_right_disabled = "arrow_right_disabled.tex",
    arrow_right_down = "crafting_inventory_arrow_r_hl.tex",
    bg_middle = "blank.tex",
    bg_middle_focus = "blank.tex",
    bg_middle_changing = "blank.tex",
    bg_end = "blank.tex",
    bg_end_focus = "blank.tex",
    bg_end_changing = "blank.tex"
}

local spinner_font = {font = UIFONT, size = 26}

local SCALE = 0.85

local FORCE_NEWTAG = false

local SkinSelector = Class(Widget, function(self, recipe, owner, skin_name)
    Widget._ctor(self, "DST Skins Selector")
    self.recipe = recipe
    self.owner = owner

    self.skins_list = self:GetSkinsList()
    self.skins_options = self:GetSkinOptions()

    self:SetScale(SCALE)

    local spinner_width = 220
    local spinner_height = 110

    -- deleted, I don't like the background, I think it doesn't fit
    if TUNING.HUDSKIN and TUNING.HUDSKIN.current then
        local _self=self
        self.spinner_bg = self:AddChild(Image("images/hud.xml", "crafting_submenu_texture.tex"))
        self.spinner_bg:SetScale(1, 1.32, 1)
        self.spinner_bg:SetPosition(0, -55)
        function self.spinner_bg:OnControl(...)
            return _self.spinner:OnControl(...)
        end
    end

    self.spinner = self:AddChild(Spinner({}, spinner_width, nil, spinner_font, nil, nil, textures, true, 250, nil))
    self.spinner.auto_shrink_text = true
    self.spinner:SetPosition(0, -spinner_height / 2, 0)
    self.spinner.fgimage:SetPosition(0, 0)
    self.spinner.fgimage:SetScale(1.2)
    self.spinner.text:SetPosition(0, -35)
    self.spinner.text:Hide()
    self.spinner.background:ScaleToSize(spinner_width + 2, spinner_height)
    self.spinner.background:SetPosition(0, 6)
    -- self.spinner:AddControllerHints(CONTROL_INVENTORY_USEONSCENE, CONTROL_INVENTORY_USEONSELF, true)

    if #self.skins_options == 1 then
        self.spinner.fgimage:SetPosition(0, 0)
        self.spinner.fgimage:SetScale(1.2)
        self.spinner.text:Hide()
    else
        self.spinner.fgimage:SetPosition(0, 15)
        self.spinner.fgimage:SetScale(1)
        self.spinner.text:Show()
    end
    -- deleted
    --[[
    self.line_top = self:AddChild(Image("images/ui.xml", "line_horizontal_white.tex"))
    self.line_top:SetPosition(0, 0)
    local BROWN = {97 / 255, 73 / 255, 46 / 255, 255 / 255} -- added
    self.line_top:SetTint(unpack(BROWN))

    self.line_bottom = self:AddChild(Image("images/ui.xml", "line_horizontal_white.tex"))
    self.line_bottom:SetPosition(0, -spinner_height)
    self.line_bottom:SetTint(unpack(BROWN))
]]
    self.focus_forward = self.spinner

    self.spinner:SetWrapEnabled(#self.skins_options > 1)
    self.spinner:SetOptions(self.skins_options)

    self.spinner:SetSelectedIndex(skin_name == nil and 1 or self:GetIndexForSkin(skin_name) or 1)

    self.widget_height = spinner_height * SCALE
end)
function SkinSelector:Refresh(recipe, skin_name)
    self.recipe = recipe
    self.skins_list = self:GetSkinsList()
    self.skins_options = self:GetSkinOptions()

    if #self.skins_options == 1 then
        self.spinner.fgimage:SetPosition(0, 0)
        self.spinner.fgimage:SetScale(1.2)
        self.spinner.text:Hide()
    else
        self.spinner.fgimage:SetPosition(0, 15)
        self.spinner.fgimage:SetScale(1)
        self.spinner.text:Show()
    end

    self.spinner:SetWrapEnabled(#self.skins_options > 1)
    self.spinner:SetOptions(self.skins_options)

    self.spinner:SetSelectedIndex(skin_name == nil and 1 or self:GetIndexForSkin(skin_name) or 1)
end
function SkinSelector:RefreshControllers(controller_mode)
    self.spinner:RefreshControllers(controller_mode)
end

function SkinSelector:GetItem()
    local which = self.spinner:GetSelectedIndex()
    if which > 1 then
        local name = self.skins_list[which - 1].item
        return name
    else
        return nil -- self.recipe.name
    end
end

function SkinSelector:GetIndexForSkin(skin)
    for i = 1, #self.skins_list do if self.skins_list[i].item == skin then return i + 1 end end
    return 1
end

function SkinSelector:SelectSkin(skin_name)
    self.spinner:SetSelectedIndex(skin_name == nil and 1 or self:GetIndexForSkin(skin_name) or 1)
end

function SkinSelector:GetSkinsList()
    local skins_list = {}
    if self.recipe and PREFAB_SKINS[self.recipe.product] then
        for _, item_type in pairs(PREFAB_SKINS[self.recipe.product]) do
            local has_item, modified_time = TheInventory:CheckOwnershipGetLatest(item_type)
            if has_item then
                local data = {}
                data.type = type
                data.item = item_type
                table.insert(skins_list, data)
            end
        end
    end
    return skins_list
end

function SkinSelector:GetSkinOptions()
    local skin_options = {}

    local non_skin_image = self.recipe.imagefn ~= nil and self.recipe.imagefn() or self.recipe.image
                               or (self.recipe.product .. ".tex")
    table.insert(skin_options, {
        text = STRINGS.UI.CRAFTING.DEFAULT,
        data = nil,
        colour = DEFAULT_SKIN_COLOR,
        new_indicator = false,
        image = {self.recipe:GetAtlas(), non_skin_image, "default.tex"}
    })
    if self.skins_list ~= nil and self.recipe.chooseskin == nil then
        for which = 1, #self.skins_list do
            local item = self.skins_list[which].item

            local colour = GetColorForItem(item)
            local text_name = GetSkinName(item)
            local image_name = GetSkinInvIconName(item) .. ".tex"
            table.insert(skin_options, {
                text = text_name,
                data = nil,
                colour = colour,
                image = {GetInventoryItemAtlas(image_name), image_name or "default.tex", "default.tex"}
            })
        end
    end

    return skin_options
end

return SkinSelector
